@echo off
title Sacrifice Games Search Comfort Tool V1.1 by Stefan Pohl, www.sp-cc.de
@echo ***************************************************************
@echo *** Sacrifice Games Search and Statistics Comfort Tool V1.1 ***
@echo ***           (C) 2022, Stefan Pohl, www.sp-cc.de           ***
@echo ***************************************************************
@echo ***     This tool uses pgn-extract (C) David J. Barnes      ***
@echo ***************************************************************
@echo ***************************************************************
@echo Enter the name of your games pgn-file (without .pgn ending!) The file must be in the working folder!
set /p filename=
set gamebase=%filename%.pgn

@echo *********************************************************************************
@echo Enter the maximum length of all non-draw games, which the tool has to investigate. For human games
@echo or adjucated engine games, at least 80 moves are recommended, for non-adjucated engine games, at
@echo least 90 or 100 is recommended. Values below 40 will be set to 80 and above 250 will be set to 250
@echo off
set /p moves=
set /A movelimit=moves
if %movelimit% LSS 40 set /A movelimit=80
if %movelimit% GEQ 250 set /A movelimit=250
@echo *********************************************************************************

REM *************************************
REM *** Start of code, do not change! ***
REM *************************************
CALL :StartTimer

@echo *** Read pgn-file: %gamebase%
copy "%gamebase%" "bin/results.pgn" >NUL
cd ./bin

CALL :countgames results.pgn
set /A numb_origin=%c_numb_pgn%

if %numb_origin% LEQ 0 (
  @echo *************************************************************************
  @echo *** ERROR - no games found in the file 
  @echo *************************************************************************
  cd ../
  goto error
)
del games_with_sacrifices.pgn 2>nul
del results_opt2.pgn 2>nul 
del results_opt3.pgn 2>nul 
del results_opt5.pgn 2>nul 
del results_opt9.pgn 2>nul 
del final_2.pgn 2>nul
del final_3.pgn 2>nul
del final_5.pgn 2>nul
del final_9.pgn 2>nul

fsutil file createnew results_opt2.pgn 0 >NUL
fsutil file createnew results_opt3.pgn 0 >NUL
fsutil file createnew results_opt5.pgn 0 >NUL
fsutil file createnew results_opt9.pgn 0 >NUL
fsutil file createnew whitewins.pgn 0 >NUL
fsutil file createnew blackwins.pgn 0 >NUL
fsutil file createnew blacksacs.pgn 0 >NUL

@echo *** Number of games found in the source file: %numb_origin%
@echo *************************************************************************
@echo *** Filtering all non-draw games, movelimit=%movelimit%
@echo *************************************************************************
REM *** filtering the white wins and black wins and put them in different files ***
pgn-extract --quiet --fixresulttags -bl15 -bu%movelimit% -Tr1-0 -D -C -N -V results.pgn --output whitewins.pgn > NUL
pgn-extract --quiet --fixresulttags -bl15 -bu%movelimit% -Tr0-1 -D -C -N -V results.pgn --output blackwins.pgn > NUL

CALL :countgames whitewins.pgn
set /A whitewon=%c_numb_pgn%
CALL :countgames blackwins.pgn
set /A blackwon=%c_numb_pgn%
set /A numb_wins=%whitewon%+%blackwon%
@echo *** Number of won games found in the source file: %numb_wins%
@echo *************************************************************************

@echo *** Searching for 2-5+ pawnunits sacs
@echo *************************************************************************
pgn-extract --quiet -yc2_pawnsac_white whitewins.pgn --output results_opt2.pgn >NUL
pgn-extract --quiet -yc2_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
copy results_opt2.pgn whitewins.pgn > NUL
copy blacksacs.pgn blackwins.pgn > NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt2.pgn blacksacs.pgn >NUL
fsutil file createnew blacksacs.pgn 0 >NUL

pgn-extract --quiet -yc3_pawnsac_white whitewins.pgn --output results_opt3.pgn >NUL
pgn-extract --quiet -yc3_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
copy results_opt3.pgn whitewins.pgn > NUL
copy blacksacs.pgn blackwins.pgn > NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt3.pgn blacksacs.pgn >NUL
fsutil file createnew blacksacs.pgn 0 >NUL

pgn-extract --quiet -yc5_pawnsac_white whitewins.pgn --output results_opt5.pgn >NUL
pgn-extract --quiet -yc5_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
copy results_opt5.pgn whitewins.pgn > NUL
copy blacksacs.pgn blackwins.pgn > NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt5.pgn blacksacs.pgn >NUL

pgn-extract --quiet -ycqueensac_white whitewins.pgn --output results_opt9.pgn >NUL
pgn-extract --quiet -ycqueensac_black blackwins.pgn --output blacksacs.pgn >NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt9.pgn blacksacs.pgn >NUL

REM *** Second filtering for exact localisation of sacs in the pgn
del 2nd_results_opt2.pgn 2>nul 
del 2nd_results_opt3.pgn 2>nul 
del 2nd_results_opt5.pgn 2>nul 
del 2nd_results_opt9.pgn 2>nul 
del 2nd_unique_opt2.pgn 2>nul 
del 2nd_unique_opt3.pgn 2>nul  
del 2nd_unique_opt5.pgn 2>nul 
del 2nd_unique_opt9.pgn 2>nul 
fsutil file createnew final_2.pgn 0 >NUL
fsutil file createnew final_3.pgn 0 >NUL
fsutil file createnew final_5.pgn 0 >NUL
fsutil file createnew final_9.pgn 0 >NUL
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
del blacksacs.pgn 2>nul

@echo *** Second filtering for exact localisation of the sacs
@echo *************************************************************************
@echo|set /p="*** Searching for sac2..."
pgn-extract --quiet -Tr1-0 results_opt2.pgn --output whitewins.pgn > NUL
pgn-extract --quiet -Tr0-1 results_opt2.pgn --output blackwins.pgn > NUL
pgn-extract --quiet -yc2_pawnsac_white_2 --markmatches spccsac2 whitewins.pgn --output 2nd_results_opt2.pgn >NUL
pgn-extract --quiet -yc2_pawnsac_black_2 --markmatches spccsac2 blackwins.pgn --output blacksacs.pgn >NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -a2nd_results_opt2.pgn blacksacs.pgn >NUL
fsutil file createnew blacksacs.pgn 0 >NUL

@echo|set /p="sac3..."
fsutil file createnew whitewins.pgn 0 >NUL
fsutil file createnew blackwins.pgn 0 >NUL
pgn-extract --quiet -Tr1-0 results_opt3.pgn --output whitewins.pgn > NUL
pgn-extract --quiet -Tr0-1 results_opt3.pgn --output blackwins.pgn > NUL
pgn-extract --quiet -yc3_pawnsac_white_2 --markmatches spccsac3 whitewins.pgn --output 2nd_results_opt3.pgn >NUL
pgn-extract --quiet -yc3_pawnsac_black_2 --markmatches spccsac3 blackwins.pgn --output blacksacs.pgn >NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -a2nd_results_opt3.pgn blacksacs.pgn >NUL
fsutil file createnew blacksacs.pgn 0 >NUL

@echo|set /p="sac5..."
fsutil file createnew whitewins.pgn 0 >NUL
fsutil file createnew blackwins.pgn 0 >NUL
pgn-extract --quiet -Tr1-0 results_opt5.pgn --output whitewins.pgn > NUL
pgn-extract --quiet -Tr0-1 results_opt5.pgn --output blackwins.pgn > NUL
pgn-extract --quiet -yc5_pawnsac_white_2 --markmatches spccsac5 whitewins.pgn --output 2nd_results_opt5.pgn >NUL
pgn-extract --quiet -yc5_pawnsac_black_2 --markmatches spccsac5 blackwins.pgn --output blacksacs.pgn >NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -a2nd_results_opt5.pgn blacksacs.pgn >NUL
fsutil file createnew blacksacs.pgn 0 >NUL

@echo sac Queen...
fsutil file createnew whitewins.pgn 0 >NUL
fsutil file createnew blackwins.pgn 0 >NUL
pgn-extract --quiet -Tr1-0 results_opt9.pgn --output whitewins.pgn > NUL
pgn-extract --quiet -Tr0-1 results_opt9.pgn --output blackwins.pgn > NUL
pgn-extract --quiet -ycqueensac_white_2 --markmatches spccsac9 whitewins.pgn --output 2nd_results_opt9.pgn >NUL
pgn-extract --quiet -ycqueensac_black_2 --markmatches spccsac9 blackwins.pgn --output blacksacs.pgn >NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -a2nd_results_opt9.pgn blacksacs.pgn >NUL
fsutil file createnew blacksacs.pgn 0 >NUL

REM *** Delete games in the gamebases with lower sacs, which already occur in higher sacs files...***
fsutil file createnew 2nd_unique_opt2.pgn 0 >NUL
fsutil file createnew 2nd_unique_opt3.pgn 0 >NUL
fsutil file createnew 2nd_unique_opt5.pgn 0 >NUL
fsutil file createnew 2nd_unique_opt9.pgn 0 >NUL

pgn-extract --quiet -D -o2nd_unique_opt9.pgn 2nd_results_opt9.pgn >NUL
pgn-extract --quiet -c2nd_results_opt9.pgn -D -o2nd_unique_opt5.pgn 2nd_results_opt5.pgn > NUL
pgn-extract --quiet -c2nd_results_opt5.pgn -D -o2nd_unique_opt3.pgn 2nd_results_opt3.pgn > NUL
pgn-extract --quiet -c2nd_results_opt3.pgn -D -o2nd_unique_opt2.pgn 2nd_results_opt2.pgn > NUL
@echo *************************************************************************
@echo *** Set sac-comments in all found games (one by one)
@echo *************************************************************************

REM *** preparation for percent progress display
set /A numb_complete=0
set /A counter_complete=0
set /A all_percent=1
CALL :countgames 2nd_unique_opt2.pgn
set /A numb_complete+=%c_numb_pgn%
set /A numb_sacs2=%c_numb_pgn%
CALL :countgames 2nd_unique_opt3.pgn
set /A numb_complete+=%c_numb_pgn%
set /A numb_sacs3=%c_numb_pgn%
CALL :countgames 2nd_unique_opt5.pgn
set /A numb_complete+=%c_numb_pgn%
set /A numb_sacs5=%c_numb_pgn%
CALL :countgames 2nd_unique_opt9.pgn
set /A numb_complete+=%c_numb_pgn%
set /A numb_sacs9=%c_numb_pgn%

@echo 0%%      10%%       20%%       30%%       40%%       50%%       60%%       70%%       80%%       90%%       100%%
@echo ---------x---------x---------x---------x---------x---------x---------x---------x---------x---------x
@echo|set /p="."

REM *** Loop, single games processing of sac2 games***

if %numb_sacs2% LEQ 0 goto :endloopsac2
set /A loopsac2_counter=1
:loopsac2

REM *** Extract next game in a single pgn-file
pgn-extract --quiet --selectonly %loopsac2_counter% 2nd_unique_opt2.pgn --output singlegame.pgn > NUL

REM *** Split game in 2 files (Tags and notation) to avoid search-collisions when movenumber is searched
pgn-extract --quiet --plylimit 0 singlegame.pgn --output singletags2.pgn > NUL
textReplace singletags2.pgn singletags.pgn "*" ""
pgn-extract --quiet --notags singlegame.pgn --output singlenotation.pgn > NUL

del work01.pgn 2>nul
REM *** Delete all carriage returns and make the game to one long string...
for /f "Usebackq Tokens=*" %%@ in ("singlenotation.pgn") do <NUL set /P "=%%@ " >>work01.pgn

REM *** sac marker string search and give movenumber back, where sac was done
set /A movenumber=0
for /f "tokens=1-2 delims=}" %%a in ('findstr /i "{ spccsac2 }" work01.pgn') do set /A movenumber=%%b 2>nul
find /i "[Result ""0-1""]" singlegame.pgn > NUL
if errorlevel 1 (
  set /A movenumber-=1
) else (
  set /A movenumber-=0
)

del work02.pgn 2>nul 
REM *** delete sac marker string
textReplace work01.pgn work02.pgn "{ spccsac2 }" " "
del work01.pgn 2>nul 

del work03.pgn 2>nul
REM *** insert new commentary in the pgn behind the move, where the sac happened
textReplace work02.pgn work03.pgn " %movenumber%. " " {SGS-tool: sac found} %movenumber%. "
del work02.pgn 2>nul

REM *** Merge tags and notation back into a valid pgn-game and then
REM *** bring game back to normal layout (with carriage returns) and merge it into result file...
del work04.pgn 2>nul
more /S singletags.pgn >work04.pgn
more work03.pgn >>work04.pgn
pgn-extract --quiet work04.pgn -afinal_2.pgn > NUL
del work03.pgn 2>nul
del work04.pgn 2>nul

REM *** One percent progress? Then print a dot
set /A counter_complete+=1
set /A old_percent=%all_percent%+1
set /A all_percent=((1000000000/%numb_complete%)*%counter_complete%)/10000000
for /L %%A in (%old_percent%,1,%all_percent%) do @echo|set /p="."  

set /A loopsac2_counter+=1

if %loopsac2_counter% GTR %numb_sacs2% goto :endloopsac2
goto :loopsac2
:endloopsac2
del singlegame.pgn 2>nul
del singletags.pgn 2>nul
del singletags2.pgn 2>nul
del singlenotation.pgn 2>nul


REM *** Loop, single games processing of sac3 games***

if %numb_sacs3% LEQ 0 goto :endloopsac3
set /A loopsac3_counter=1
:loopsac3

REM *** Extract next game in a single pgn-file
pgn-extract --quiet --selectonly %loopsac3_counter% 2nd_unique_opt3.pgn --output singlegame.pgn > NUL

REM *** Split game in 2 files (Tags and notation) to avoid search-collisions when movenumber is searched
pgn-extract --quiet --plylimit 0 singlegame.pgn --output singletags2.pgn > NUL
textReplace singletags2.pgn singletags.pgn "*" ""
pgn-extract --quiet --notags singlegame.pgn --output singlenotation.pgn > NUL

del work01.pgn 2>nul
REM *** Delete all carriage returns and make the game to one long string...
for /f "Usebackq Tokens=*" %%@ in ("singlenotation.pgn") do <NUL set /P "=%%@ " >>work01.pgn

REM *** sac marker string search and give movenumber back, where sac was done
set /A movenumber=0
for /f "tokens=1-2 delims=}" %%a in ('findstr /i "{ spccsac3 }" work01.pgn') do set /A movenumber=%%b 2>nul
find /i "[Result ""0-1""]" singlegame.pgn > NUL
if errorlevel 1 (
  set /A movenumber-=2
) else (
  set /A movenumber-=1
)

del work02.pgn 2>nul 
REM *** delete sac marker string
textReplace work01.pgn work02.pgn "{ spccsac3 }" " "
del work01.pgn 2>nul 

del work03.pgn 2>nul
REM *** insert new commentary in the pgn behind the move, where the sac happened
textReplace work02.pgn work03.pgn " %movenumber%. " " {SGS-tool: sac found} %movenumber%. "
del work02.pgn 2>nul

REM *** Merge tags and notation back into a valid pgn-game and then
REM *** bring game back to normal layout (with carriage returns) and merge it into result file...
del work04.pgn 2>nul
more /S singletags.pgn >work04.pgn
more work03.pgn >>work04.pgn
pgn-extract --quiet work04.pgn -afinal_3.pgn > NUL
del work03.pgn 2>nul
del work04.pgn 2>nul

set /A loopsac3_counter+=1

REM *** One percent progress? Then print a dot
set /A counter_complete+=1
set /A old_percent=%all_percent%+1
set /A all_percent=((1000000000/%numb_complete%)*%counter_complete%)/10000000
for /L %%A in (%old_percent%,1,%all_percent%) do @echo|set /p="."  

if %loopsac3_counter% GTR %numb_sacs3% goto :endloopsac3
goto :loopsac3
:endloopsac3
del singlegame.pgn 2>nul
del singletags.pgn 2>nul
del singletags2.pgn 2>nul
del singlenotation.pgn 2>nul


REM *** Loop, single games processing of sac5 games***

if %numb_sacs5% LEQ 0 goto :endloopsac5
set /A loopsac5_counter=1
:loopsac5

REM *** Extract next game in a single pgn-file
pgn-extract --quiet --selectonly %loopsac5_counter% 2nd_unique_opt5.pgn --output singlegame.pgn > NUL

REM *** Split game in 2 files (Tags and notation) to avoid search-collisions when movenumber is searched
pgn-extract --quiet --plylimit 0 singlegame.pgn --output singletags2.pgn > NUL
textReplace singletags2.pgn singletags.pgn "*" ""
pgn-extract --quiet --notags singlegame.pgn --output singlenotation.pgn > NUL

del work01.pgn 2>nul
REM *** Delete all carriage returns and make the game to one long string...
for /f "Usebackq Tokens=*" %%@ in ("singlenotation.pgn") do <NUL set /P "=%%@ " >>work01.pgn

REM *** sac marker string search and give movenumber back, where sac was done
set /A movenumber=0
for /f "tokens=1-2 delims=}" %%a in ('findstr /i "{ spccsac5 }" work01.pgn') do set /A movenumber=%%b 2>nul
find /i "[Result ""0-1""]" singlegame.pgn > NUL
if errorlevel 1 (
  set /A movenumber-=3
) else (
  set /A movenumber-=2
)

del work02.pgn 2>nul 
REM *** delete sac marker string
textReplace work01.pgn work02.pgn "{ spccsac5 }" " "
del work01.pgn 2>nul 

del work03.pgn 2>nul
REM *** insert new commentary in the pgn behind the move, where the sac happened
textReplace work02.pgn work03.pgn " %movenumber%. " " {SGS-tool: sac found} %movenumber%. "
del work02.pgn 2>nul

REM *** Merge tags and notation back into a valid pgn-game and then
REM *** bring game back to normal layout (with carriage returns) and merge it into result file...
del work04.pgn 2>nul
more /S singletags.pgn >work04.pgn
more work03.pgn >>work04.pgn
pgn-extract --quiet work04.pgn -afinal_5.pgn > NUL
del work03.pgn 2>nul
del work04.pgn 2>nul

set /A loopsac5_counter+=1

REM *** One percent progress? Then print a dot
set /A counter_complete+=1
set /A old_percent=%all_percent%+1
set /A all_percent=((1000000000/%numb_complete%)*%counter_complete%)/10000000
for /L %%A in (%old_percent%,1,%all_percent%) do @echo|set /p="."  

if %loopsac5_counter% GTR %numb_sacs5% goto :endloopsac5
goto :loopsac5
:endloopsac5
del singlegame.pgn 2>nul
del singletags.pgn 2>nul
del singletags2.pgn 2>nul
del singlenotation.pgn 2>nul


REM *** Loop, single games processing of sacQ games***

if %numb_sacs9% LEQ 0 goto :endloopsac9
set /A loopsac9_counter=1
:loopsac9

REM *** Extract next game in a single pgn-file
pgn-extract --quiet --selectonly %loopsac9_counter% 2nd_unique_opt9.pgn --output singlegame.pgn > NUL

REM *** Split game in 2 files (Tags and notation) to avoid search-collisions when movenumber is searched
pgn-extract --quiet --plylimit 0 singlegame.pgn --output singletags2.pgn > NUL
textReplace singletags2.pgn singletags.pgn "*" ""
pgn-extract --quiet --notags singlegame.pgn --output singlenotation.pgn > NUL

del work01.pgn 2>nul
REM *** Delete all carriage returns and make the game to one long string...
for /f "Usebackq Tokens=*" %%@ in ("singlenotation.pgn") do <NUL set /P "=%%@ " >>work01.pgn

REM *** sac marker string search and give movenumber back, where sac was done
set /A movenumber=0
for /f "tokens=1-2 delims=}" %%a in ('findstr /i "{ spccsac9 }" work01.pgn') do set /A movenumber=%%b 2>nul
find /i "[Result ""0-1""]" singlegame.pgn > NUL
if errorlevel 1 (
  set /A movenumber-=3
) else (
  set /A movenumber-=2
)

del work02.pgn 2>nul 
REM *** delete sac marker string
textReplace work01.pgn work02.pgn "{ spccsac9 }" " "
del work01.pgn 2>nul 

del work03.pgn 2>nul
REM *** insert new commentary in the pgn behind the move, where the sac happened
textReplace work02.pgn work03.pgn " %movenumber%. " " {SGS-tool: Queen sac found} %movenumber%. "
del work02.pgn 2>nul

REM *** Merge tags and notation back into a valid pgn-game and then
REM *** bring game back to normal layout (with carriage returns) and merge it into result file...
del work04.pgn 2>nul
more /S singletags.pgn >work04.pgn
more work03.pgn >>work04.pgn
pgn-extract --quiet work04.pgn -afinal_9.pgn > NUL
del work03.pgn 2>nul
del work04.pgn 2>nul

set /A loopsac9_counter+=1

REM *** One percent progress? Then print a dot
set /A counter_complete+=1
set /A old_percent=%all_percent%+1
set /A all_percent=((1000000000/%numb_complete%)*%counter_complete%)/10000000
for /L %%A in (%old_percent%,1,%all_percent%) do @echo|set /p="."  

if %loopsac9_counter% GTR %numb_sacs9% goto :endloopsac9
goto :loopsac9
:endloopsac9
del singlegame.pgn 2>nul
del singletags.pgn 2>nul
del singletags2.pgn 2>nul
del singlenotation.pgn 2>nul

@echo done
@echo *************************************************************************

REM *** Count the number of games in the result bases...***
CALL :countgames final_9.pgn
set /A numb_opt9=%c_numb_pgn%
CALL :countgames final_5.pgn
set /A numb_opt5=%c_numb_pgn%
CALL :countgames final_3.pgn
set /A numb_opt3=%c_numb_pgn%
CALL :countgames final_2.pgn
set /A numb_opt2=%c_numb_pgn%

REM *** Count the number off all sacs found (add all sac-numbers) ***
set /A numb_sum=%numb_opt2%+%numb_opt3%+%numb_opt5%+%numb_opt9%

cd ../
fsutil file createnew statistics.txt 0 >NUL

REM *** Build the statistics.txt file ***
@echo *************************************************************** >statistics.txt
@echo *** Sacrifice Games Search and Statistics Comfort Tool V1.1 *** >>statistics.txt
@echo ***           (C) 2022, Stefan Pohl, www.sp-cc.de           *** >>statistics.txt
@echo *************************************************************** >>statistics.txt
@echo *** Sacrifice-statistics of the file %gamebase% >>statistics.txt
@echo *************************************************************** >>statistics.txt
@echo ************************************************************************************* >>statistics.txt
@echo Number of games found in the source file: %numb_origin% >>statistics.txt
@echo ************************************************************************************* >>statistics.txt
@echo Number of won games found in the source file: %numb_wins% >>statistics.txt
@echo ************************************************************************************* >>statistics.txt
@echo Max. allowed length of games, investigated: %movelimit% moves >>statistics.txt
@echo ************************************************************************************* >>statistics.txt
CALL :percent %numb_origin% %numb_opt9%
set percent_overall=%percent%
CALL :percent %numb_wins% %numb_opt9%
set percent_of_wins=%percent%
@echo Number of Queen sacrifices (5+)  : %numb_opt9% (%percent_overall% of all games) (%percent_of_wins% of won games) >>statistics.txt

CALL :percent %numb_origin% %numb_opt5%
set percent_overall=%percent%
CALL :percent %numb_wins% %numb_opt5%
set percent_of_wins=%percent%
@echo Number of 5+ pawnunits sacrifices: %numb_opt5% (%percent_overall% of all games) (%percent_of_wins% of won games) >>statistics.txt

CALL :percent %numb_origin% %numb_opt3%
set percent_overall=%percent%
CALL :percent %numb_wins% %numb_opt3%
set percent_of_wins=%percent%
@echo Number of 3 pawnunits sacrifices : %numb_opt3% (%percent_overall% of all games) (%percent_of_wins% of won games) >>statistics.txt

CALL :percent %numb_origin% %numb_opt2%
set percent_overall=%percent%
CALL :percent %numb_wins% %numb_opt2%
set percent_of_wins=%percent%
@echo Number of 2 pawnunits sacrifices : %numb_opt2% (%percent_overall% of all games) (%percent_of_wins% of won games) >>statistics.txt

CALL :percent %numb_origin% %numb_sum%
set percent_overall=%percent%
CALL :percent %numb_wins% %numb_sum%
set percent_of_wins=%percent%
@echo Number of all sacrifices         : %numb_sum% (%percent_overall% of all games) (%percent_of_wins% of won games) >>statistics.txt
@echo ************************************************************************************* >>statistics.txt
@echo *** Mention: Games containing more than one sac can lead to mismatches in the sac-localisation! >>statistics.txt
@echo *** And in each game only one sac can be marked in the comments. >>statistics.txt
@echo *** Mention: The regular SGS-tool and the EAS-tool will find around +50%% more sac-games! >>statistics.txt
@echo *** (Additionally to the one pawn sacs, which the SGS comfort tool ignores) >>statistics.txt
@echo **************************************************** >>statistics.txt
@echo *** SGS-tool (C) 2022 Stefan Pohl (www.sp-cc.de) *** >>statistics.txt
@echo **************************************************** >>statistics.txt

del sacgames_1_pawns.pgn 2>nul
del sacgames_2_pawns.pgn 2>nul
del sacgames_3_pawns.pgn 2>nul
del sacgames_4_pawns.pgn 2>nul
del sacgames_5_pawns.pgn 2>nul
del sacgames_queensacs.pgn 2>nul

cd ./bin
REM *** Build final output pgn-files ***
pgn-extract --quiet -o../sacgames_2_pawns.pgn final_2.pgn >NUL
pgn-extract --quiet -o../sacgames_3_pawns.pgn final_3.pgn >NUL
pgn-extract --quiet -o../sacgames_5_pawns.pgn final_5.pgn >NUL
pgn-extract --quiet -o../sacgames_queensacs.pgn final_9.pgn >NUL


REM *** delete all working files ***
del results.pgn 2>nul
del results_opt2.pgn 2>nul 
del results_opt3.pgn 2>nul 
del results_opt5.pgn 2>nul 
del results_opt9.pgn 2>nul  
del 2nd_results_opt2.pgn 2>nul 
del 2nd_results_opt3.pgn 2>nul 
del 2nd_results_opt5.pgn 2>nul 
del 2nd_results_opt9.pgn 2>nul 
del 2nd_unique_opt2.pgn 2>nul 
del 2nd_unique_opt3.pgn 2>nul  
del 2nd_unique_opt5.pgn 2>nul 
del 2nd_unique_opt9.pgn 2>nul 
del final_2.pgn 2>nul
del final_3.pgn 2>nul
del final_5.pgn 2>nul
del final_9.pgn 2>nul
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
del blacksacs.pgn 2>nul
del singlegame.pgn 2>nul

cd ../
@echo *** See the statistics.txt file for statistics
@echo *******************************************************
@echo *** Mention: Games containing more than one sac can lead to mismatches in the sac-localisation!
@echo *** And in each game only one sac can be marked in the comments.
@echo *** Mention: The regular SGS-tool and the EAS-tool will find around +50%% more sac-games!
@echo *** (Additionally to the one pawn sacs, which the SGS comfort tool ignores)
@echo **************************************************************************
@echo *** Games stored in 4 files: 
@echo *** sacgames_(2,3,5)_pawns.pgn and sacgames_queensacs.pgn
:jumpover
@echo **************************************************************************
CALL :StopTimer
CALL :DisplayTimerResult
@echo **************************************************************************
:error
CALL ./bin/playwav %windir%/media/alarm02.wav
pause

goto :EOF

:countgames
set /A c_numb_pgn=0
for /f "tokens=2,* delims= " %%G in ('find /C "[White " %~1') do set /A c_numb_pgn=%%H
exit /B 0

:percent
if %~1 LEQ 0 (
set percent=00.00%%
set /A percentx100=0
EXIT /B 0
)
set /A l_base=((1000000000/%~1)*%~2)
set /A l_percent=%l_base%/1000000
set /A l_rest1=%l_percent%%%10
set /A l_percent=%l_percent%/10
set /A l_secdig_pc=%l_base%/100000
set /A l_rest2=%l_secdig_pc%%%10
set /A l_thirddig_pc=%l_base%/10000
set /A l_rest3=%l_thirddig_pc%%%10

REM *** Round 2nd digit up, if 3rd digit is GEQ 5 and round 1st digit and percent-value up, if necessary
if %l_rest3% GEQ 5 set /A l_rest2+=1
if %l_rest2% GEQ 10 (
  set /A l_rest2-=10
  set /A l_rest1+=1
)
if %l_rest1% GEQ 10 (
  set /A l_rest1-=10
  set /A l_percent+=1
)

if %l_percent% LSS 10 (
  set percent=0%l_percent%.%l_rest1%%l_rest2%%%
) else (
  set percent=%l_percent%.%l_rest1%%l_rest2%%%
)
set /A percentx100=(%l_percent%*100)+(%l_rest1%*10)+%l_rest2%

if %l_percent% GTR 99 (
set percent=100.0%%
set /A percentx100=10000
)
EXIT /B 0

:StartTimer
set StartTIME=%TIME%
for /f "usebackq tokens=1-4 delims=:., " %%f in (`echo %StartTIME: =0%`) do set /a Start100S=1%%f*360000+1%%g*6000+1%%h*100+1%%i-36610100
EXIT /B 0

:StopTimer
set StopTIME=%TIME%
for /f "usebackq tokens=1-4 delims=:., " %%f in (`echo %StopTIME: =0%`) do set /a Stop100S=1%%f*360000+1%%g*6000+1%%h*100+1%%i-36610100
if %Stop100S% LSS %Start100S% set /a Stop100S+=8640000
set /a TookTime=%Stop100S%-%Start100S%
set TookTimePadded=0%TookTime%
EXIT /B 0

:DisplayTimerResult
set /A t_elapsed=%TookTime:~0,-2%.%TookTimePadded:~-2% 2>nul
set /A t_seconds=%t_elapsed%
set /A t_minutes=%t_seconds%/60
set /A t_hours=%t_minutes%/60
set /A t_seconds-=%t_minutes%*60
set /A t_minutes-=%t_hours%*60
if %t_seconds% LEQ 9 set t_seconds=0%t_seconds%
if %t_minutes% LEQ 9 set t_minutes=0%t_minutes%
if %t_hours% LEQ 9 set t_hours=0%t_hours%
@echo *** Elapsed time: %t_hours%:%t_minutes%:%t_seconds%
EXIT /B 0
